"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const cron_validator_1 = require("cron-validator");
const cronstrue_1 = tslib_1.__importDefault(require("cronstrue"));
const logger_1 = tslib_1.__importDefault(require("../utilities/logger"));
const utilities_1 = tslib_1.__importDefault(require("../utilities/utilities"));
/*
* this Commands add support for cron jobs
*  */
class Cron extends command_1.Command {
    // only 2 parameters required HASH_TYPE and INPUT_STRING
    async run() {
        const { args, flags } = this.parse(Cron);
        args.string = utilities_1.default.getInputStringFromCmd(this, flags, args); // from either -s or args
        args.action = this.getAction(flags, args); //by default let it be sha1
        //check params after evaluating all
        this.checkParameters(flags, args);
        this.evalCron(flags, args);
    }
    // to check required parameters passed or not
    // tslint:disable-next-line:no-unused
    checkParameters(flags, args) {
        if (args.string === undefined || args.string === '')
            logger_1.default.error(this, 'Input string is empty or undefined');
        if (!cron_validator_1.isValidCron(args.string)) {
            logger_1.default.error(this, `Invalid Cron expression : ${chalk_1.default.red(args.string)}`);
        }
        if (args.action === undefined || args.string === '')
            logger_1.default.error(this, 'Action empty or undefined');
    }
    // tslint:disable-next-line:no-unused
    evalCron(flags, args) {
        // Logger.success(this, `Action: ${chalk.green(args.action)}`)
        if (args.action === Cron.DESCRIBE) {
            let output = cronstrue_1.default.toString(args.string);
            logger_1.default.success(this, output);
        }
        else if (args.action === Cron.RUN) {
            logger_1.default.success(this, 'run command, coming soon...');
        }
    }
    // tslint:disable-next-line:no-unused
    getAction(flags, args) {
        if (flags.describe) // find  human readable descriptions for cron
            return Cron.DESCRIBE;
        else if (flags.run) // if run is given
            return Cron.RUN;
        logger_1.default.error(this, 'Invalid Or Unsupported action');
    }
}
exports.default = Cron;
Cron.description = 'Cron Expressions helper and scheduler';
Cron.RUN = 'run';
Cron.DESCRIBE = 'desc';
Cron.flags = {
    help: command_1.flags.help({ char: 'h' }),
    string: command_1.flags.string({ char: 's', description: 'Cron expression' }),
    describe: command_1.flags.boolean({ char: 'd', description: 'Describe cron expressions into human readable descriptions' }),
    run: command_1.flags.boolean({ char: 'r', description: 'run command using cron expression' }),
};
Cron.args = [{ name: 'string' }];
//# sourceMappingURL=cron.js.map